@@ -1,11 +1,13 @@
-import csv
 import datetime
 import logging
 from time import sleep
 
 import requests
 
-from ravenpackapi.exceptions import api_method, APIException, DataFileTimeout
+from ravenpackapi.exceptions import (api_method,
+                                     APIException,
+                                     DataFileTimeout,
+                                     JobNotProcessing)
 from ravenpackapi.util import to_curl, parse_csv_line
 
 logger = logging.getLogger(__name__)
@@ -51,11 +53,24 @@ class Job(object):
         token = self.token
         response = self.api.request(
             endpoint="/jobs/%s" % token,
-            data={
+            json={
                 "token": token,
             },
         )
-        self._data.update(response.json())
+        json = response.json()
+        self._data.update(json)
+        return json.get('status')
+
+    @api_method
+    def cancel(self):
+        token = self.token
+        response = self.api.request(
+            endpoint="/jobs/%s" % token,
+            method='delete'
+        )
+        response.json()
+        self._data.update({"status": 'cancelled'})
+        return self.status
 
     @api_method
     def wait_for_completion(self, timeout_seconds=None):
@@ -70,6 +85,9 @@ class Job(object):
         while True:
             if self.is_ready:
                 break
+            if not self.is_processing:
+                raise JobNotProcessing("The job went in an error state: %s" % self.status,
+                                       status=self.status)
             sleep(self._FILE_AVAILABILIY_SECONDS_DELAY)
             try:
                 self.get_status()
@@ -97,6 +115,7 @@ class Job(object):
             response = requests.get(job.url,
                                     headers=api.headers,
                                     stream=True,
+                                    **api.common_request_params
                                     )
             if response.status_code != 200:
                 logger.error("Error calling the API, we tried: %s" % to_curl(response.request))
@@ -109,7 +128,7 @@ class Job(object):
                     output.write(chunk)
 
     @api_method
-    def iterate_results(self):
+    def iterate_results(self, include_headers=False):
         api = self.api
         job = self  # just to be clear
         job.wait_for_completion()
@@ -118,10 +137,15 @@ class Job(object):
             r = s.get(job.url,
                       headers=api.headers,
                       stream=True,
+                      **api.common_request_params
                       )
             iterator = r.iter_lines(chunk_size=self._CHUNK_SIZE)
+
             headers = next(iterator)  # discard the headers
 
+            if include_headers:
+                yield parse_csv_line(headers)
+
             for line in iterator:
                 fields = parse_csv_line(line)
                 yield fields
